=== modified file 'NEWS'
--- NEWS	2006-12-19 08:26:36 +0000
+++ NEWS	2006-12-19 08:45:03 +0000
@@ -66,6 +66,11 @@
 
     * Test suite ends cleanly on Windows.  (Vincent Ladeuil)
 
+    * Attribute 'encoding_type' of class Command equals to 'exact' 
+      force sys.stdout to be binary stream on Windows and therefore
+      keep exact line-endings (without LF -> CRLF conversion).
+      (Alexander Belchenko)
+
   BUG FIXES:
 
     * ``bzr annotate`` now defaults to showing dotted revnos for merged

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-12-19 08:26:36 +0000
+++ bzrlib/builtins.py	2006-12-19 08:45:03 +0000
@@ -1218,7 +1218,6 @@
     takes_options = ['revision', 'diff-options', 'prefix']
     aliases = ['di', 'dif']
     encoding_type = 'exact'
-    binary_stdout = True
 
     @display_command
     def run(self, revision=None, file_list=None, diff_options=None,
@@ -1751,7 +1750,7 @@
 
     takes_options = ['revision', 'name-from-revision']
     takes_args = ['filename']
-    binary_stdout = True
+    encoding_type = 'exact'
 
     @display_command
     def run(self, filename, revision=None, name_from_revision=False):

=== modified file 'bzrlib/bundle/commands.py'
--- bzrlib/bundle/commands.py	2006-12-13 08:19:49 +0000
+++ bzrlib/bundle/commands.py	2006-12-19 08:45:03 +0000
@@ -98,7 +98,7 @@
                             type=unicode)]
     takes_args = ['base?']
     aliases = ['bundle']
-    binary_stdout = True
+    encoding_type = 'exact'
 
     def run(self, base=None, revision=None, output=None, remember=False):
         from bzrlib import user_encoding

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-12-19 08:26:36 +0000
+++ bzrlib/commands.py	2006-12-19 08:45:03 +0000
@@ -214,18 +214,16 @@
             replace - put in a bogus character (typically '?')
             exact - do not encode sys.stdout
 
-    binary_stdout
-        Typically output of Command use native line-endings on each
-        platform. This leads to problems on win32 with some commands,
-        because Windows convert LF to CRLF.
-        If this attribute is True then output forced to binary
-        (without such conversion) on Windows
+            NOTE: by default on Windows sys.stdout opened as text stream,
+            therefore line-endings LF converted to CRLF.
+            When command use encoding_type = 'exact' then
+            sys.stdout forced to be binary stream and line-endings
+            will not mangled.
     """
     aliases = []
     takes_args = []
     takes_options = []
     encoding_type = 'strict'
-    binary_stdout = False
 
     hidden = False
     
@@ -250,14 +248,15 @@
         """Return a file linked to stdout, which has proper encoding."""
         assert self.encoding_type in ['strict', 'exact', 'replace']
 
-        if sys.platform == 'win32' and self.binary_stdout:
-            if hasattr(sys.stdout, 'fileno'):
-                import msvcrt
-                msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
-
         # Originally I was using self.stdout, but that looks
         # *way* too much like sys.stdout
         if self.encoding_type == 'exact':
+            # force sys.stdout to be binary stream on win32
+            if sys.platform == 'win32':
+                fileno = getattr(sys.stdout, 'fileno', None)
+                if fileno:
+                    import msvcrt
+                    msvcrt.setmode(fileno(), os.O_BINARY)
             self.outf = sys.stdout
             return
 

